#! /bin/sh -e

srcdir=$1

(
$srcdir/mkinstalldirs include/sys
cp /usr/include/sys/types.h include/sys/types.h
patch include/sys/types.h <<EOF
--- /usr/include/sys/types.h	Tue Sep  1 12:07:45 1998
+++ types.h	Wed Oct 18 11:28:15 2000
@@ -13,7 +13,9 @@
 #ifndef _SYS_TYPES_H
 #define	_SYS_TYPES_H
 
+#ifndef __GNUC__
 #pragma ident	"@(#)types.h	1.63	98/04/28 SMI"
+#endif
 
 #include <sys/isa_defs.h>
 #include <sys/feature_tests.h>
@@ -49,6 +51,9 @@
  * (eg, GNU C) that don't support long long, and need to include
  * this header file.
  */
+
+#if !defined(IN_CK) && !defined(_CARBONKERNEL_)
+
 #if __STDC__ - 0 == 0 && !defined(_NO_LONGLONG)
 typedef	long long		longlong_t;
 typedef	unsigned long long	u_longlong_t;
@@ -64,6 +69,13 @@
 } u_longlong_t;
 #endif	/* __STDC__ - 0 == 0 && !defined(_NO_LONGLONG) */
 
+#else /* IN_CK || _CARBONKERNEL_ */
+
+typedef	long long		longlong_t;
+typedef	unsigned long long	u_longlong_t;
+
+#endif /* !IN_CK && !_CARBONKERNEL_ */
+
 /*
  * These types (t_{u}scalar_t) exist because the XTI/TPI/DLPI standards had
  * to use them instead of int32_t and uint32_t because DEC had
@@ -338,6 +350,8 @@
 
 typedef	uid_t	gid_t;			/* GID type		*/
 
+#if !defined(IN_CK) && !defined(_CARBONKERNEL_)
+
 /*
  * POSIX definitions are same as defined in thread.h and synch.h.
  * Any changes made to here should be reflected in corresponding
@@ -418,6 +432,65 @@
 	void	*__pthread_rwlockattrp;
 } pthread_rwlockattr_t;
 
+#else /* IN_CK || _CARBONKERNEL_ */
+
+struct rtl_cleanup_struct {
+
+    void (*routine)(void *arg);
+    void *arg;
+    struct rtl_cleanup_struct *next;
+};
+
+struct rtl_thread_struct {
+
+    unsigned long handle;
+    struct rtl_cleanup_struct *cleanup;
+};
+
+typedef struct rtl_thread_struct RTL_THREAD_STRUCT;
+
+typedef struct rtl_thread_struct *pthread_t;
+
+struct rtl_sched_param {
+
+    int sched_priority;
+};
+
+#define sched_param rtl_sched_param
+
+enum rtl_task_states {
+
+    RTL_THREAD_READY =1,
+    RTL_THREAD_DELAYED =2,
+    RTL_THREAD_DORMANT =4,
+    RTL_THREAD_SUSPENDED =8,
+    RTL_THREAD_ACTIVE =16,
+    RTL_THREAD_ZOMBIE =32,
+    RTL_THREAD_WAIT_NEEDED =64
+};
+
+#ifndef _SIZE_T
+#define	_SIZE_T
+#if defined(_LP64) || defined(_I32LPx)
+typedef	ulong_t	size_t;		/* size of something in bytes */
+#else
+typedef	uint_t	size_t;		/* (historical version) */
+#endif
+#endif	/* _SIZE_T */
+
+typedef struct STRUCT_PTHREAD_ATTR {
+
+    size_t stack_size;
+    struct rtl_sched_param sched_param;
+    int cpu;
+    int use_fp;
+    int detachstate;
+    enum rtl_task_states initial_state;
+
+} pthread_attr_t;
+
+#endif /* !IN_CK && !_CARBONKERNEL_ */
+
 typedef ulong_t	dev_t;			/* expanded device type */
 
 #if defined(_LP64) || defined(_I32LPx)
@@ -458,7 +531,11 @@
 
 #ifndef _CLOCKID_T
 #define	_CLOCKID_T
+#if !defined(IN_CK) && !defined(_CARBONKERNEL_)
 typedef	int	clockid_t;	/* clock identifier type */
+#else
+typedef struct rtl_clock *clockid_t;
+#endif /* !IN_CK && !_CARBONKERNEL_ */
 #endif	/* ifndef _CLOCKID_T */
 
 #ifndef _TIMER_T
EOF
$srcdir/mkinstalldirs include/rtl/sys
cp include/sys/types.h include/rtl/sys/types.h
) > /dev/null

echo rtl/sys/types.h

exit 0
