/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: POSIX signal header replacement for the RTLinux model
 * on top of CarbonKernel.
 * Based on posix/signal.h from the standard RTLinux distribution
 * written by Michael Barabanov <baraban@fsmlabs.com>.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _posix_signal_h
#define _posix_signal_h

#define SA_FOCUS  0x00000008
#define SA_IRQ	  0x00000010

typedef unsigned long rtl_sigset_t;

#define rtl_sigaddset(set, sig) set_bit(sig, set)
#define rtl_sigdelset(set, sig) clear_bit(sig, set)
#define rtl_sigismember(set, sig) test_bit(sig, set)
#define rtl_sigemptyset(set) do { *(set) = 0; } while (0)
#define rtl_sigfillset(set) do { *(set) = ~0; } while (0)

struct rtl_siginfo {
    int junk;
};

struct rtl_sigaction {
    union {
	void (*_sa_handler)(int);
	void (*_sa_sigaction)(int, struct rtl_siginfo *, void *);
    } _u;
    int sa_flags;
    unsigned long sa_focus;
    rtl_sigset_t sa_mask;
};

#define sa_sigaction _u._sa_sigaction
#define sa_handler   _u._sa_handler
#define sigaction    rtl_sigaction

#define RTL_SIGIRQMIN      256
#define RTL_SIGLOCALIRQMIN 512
#define RTL_SIGIRQMAX      1024

#ifdef __cplusplus
extern "C" {
#endif

#define sigaction ck_sigaction
int ck_sigaction(int sig,
		 const struct sigaction *act,
		 struct sigaction *oact);

#ifdef __cplusplus
}
#endif

#endif /* !_posix_signal_h */
