/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Interface to the RTLinux IPC routines.
 * Based on rt_ipc.h written by Jerry Epplin.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtl_rt_ipc_h
#define _rtl_rt_ipc_h

#include <ck/stypes.h>
#include <rtl/rtl_sema.h>
#include <rtl/rtl_compat.h>

typedef enum {

    RT_SEM_BINARY,
    RT_SEM_COUNTING

} RT_SEMTYPE;

/* Ck-specific: IPC package semaphores are now based on
   the standard semaphore implementation of RTLinux v3. */

typedef sem_t rt_sem_t;

typedef enum {

    RT_MQ_NORMAL,
    RT_MQ_URGENT

} RT_MQ_PRIO;

typedef enum {

    RT_MQ_FULL,
    RT_MQ_EMPTY,
    RT_MQ_NEITHER

} RT_MQ_STATUS;

typedef struct {

    ckhandle_t handle;

} rt_mq_t;

typedef struct {

    pthread_t thread;

} RT_TASK_IPC;

#define MAKE_RT_TASK(task) (&((task)->thread))

#define RELATIVE_TIME(d)   (rt_get_time() + (d))
#define RT_WAIT_FOREVER	   (-1)
#define RT_NO_WAIT	   0

#ifdef __cplusplus
extern "C" {
#endif

int rt_sem_init(rt_sem_t *sem,
		RT_SEMTYPE type,
		int init_val);

int rt_sem_destroy(rt_sem_t *sem);

int rt_sem_post(rt_sem_t *sem);

int rt_sem_wait(rt_sem_t *sem,
		RTIME timeout);

int rt_sem_trywait(rt_sem_t *sem);

int rt_task_delay(RTIME duration);

int rt_task_ipc_init(RT_TASK_IPC *task,
		     void (*fn)(int data),
		     int data,
		     int stack_size,
		     int priority);

int rt_task_ipc_delete(RT_TASK_IPC *task);

int rt_mq_init(rt_mq_t *mq,
	       int max_msgs,
	       int msg_size);

int rt_mq_destroy(rt_mq_t *mq);

int rt_mq_send(rt_mq_t *mq,
	       char *msg,
	       RT_MQ_PRIO prio,
	       RTIME wait);

int rt_mq_receive(rt_mq_t *mq,
		  char *msg,
		  RTIME wait);

int rtf_ipc_create(unsigned int fifo,
		   int size,
		   int rtl_to_linux);

int rtf_ipc_destroy(unsigned int fifo);

int rtf_receive(unsigned int fifo,
		void *buf,
		int count,
		RTIME timeout);

int rtf_send(unsigned int fifo,
	     void *buf,
	     int count,
	     RTIME timeout);

#ifdef __cplusplus
};
#endif

#endif /* !_rtl_rt_ipc_h */
