/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Backward compatibility header to RTL v1.x.
 * Based on rtl_compat.h from the standard RTLinux distribution
 * written by Michael Barabanov <baraban@fsmlabs.com>.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtl_compat_h
#define _rtl_compat_h

#include <rtl/rtl_sched.h>
#include <rtl/posix/time.h>
#include <rtl/posix/errno.h>

typedef pthread_t RT_TASK;

#define RT_TICKS_PER_SEC   ((long long)1193180)
#define RT_TIME_END        (HRTIME_INFINITY / 838)

#define rt_get_time()      HRT_TO_8254(gethrtime())
#define rt_delay(x)        rtl_delay((x) * 838)
#define rt_task_wait(void) (pthread_wait_np(), 0)

#ifdef __cplusplus
extern "C" {
#endif

static inline hrtime_t rtl_set_periodic_mode (hrtime_t period) {

    clockid_t clock = rtl_getschedclock();

    int rc = rtl_setclockmode (clock,
			       RTL_CLOCK_MODE_PERIODIC,
			       HRT_FROM_8254(period));

    return !rc ? HRT_TO_8254(clock_gethrtime(clock)) : RT_TIME_END;
}

static inline int rtl_set_oneshot_mode (void) {
    return rtl_setclockmode(rtl_getschedclock(),RTL_CLOCK_MODE_ONESHOT,0);
}

static inline int rt_task_use_fp (RT_TASK *task, int flag) {
    return -pthread_setfp_np (*task,flag);
}

static inline int rt_task_suspend (RT_TASK *task) {
    return -pthread_suspend_np(*task);
}

static inline int rt_task_wakeup (RT_TASK *task) {
    return -pthread_wakeup_np(*task);
}

static inline int rt_task_delete (RT_TASK *task) {
    return pthread_delete_np(*task) ? -EINVAL : 0;
}

static inline int rt_task_make_periodic (RT_TASK *task,
					 RTIME start_time,
					 RTIME period) {

    return -pthread_make_periodic_np(*task,
				     HRT_FROM_8254(start_time),
				     HRT_FROM_8254(period));
}

static inline int rt_task_init (RT_TASK *task,
				void (*fn)(int data),
				int data,
				int stack_size,
				int priority) {
    struct rtl_sched_param param;
    pthread_attr_t attr;

    pthread_attr_init(&attr);
    attr.initial_state = RTL_THREAD_DORMANT;
    pthread_attr_setstacksize(&attr,stack_size);
    param.sched_priority = sched_get_priority_max(0) - priority;
    pthread_attr_setschedparam(&attr,&param);

    return -pthread_create(task,&attr,(void *(*)(void *))fn,(void *)data);
}

#ifdef __cplusplus
};
#endif

#define rtl_init(a,b,c,d,e) { rt_task_init(a,b,c,d,e); }

#define rtl_fpinit(a,b,c,d,e) { rt_task_init(a,b,c,d,e); \
                                rtl_task_use_fp(a, 1); }

#endif /* !_rtl_compat_h */
