/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Interface to the mutex and condition variable objects.
 * Based on rtl_mutex.h from the standard RTLinux distribution
 * written by Michael Barabanov <baraban@fsmlabs.com>.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtl_mutex_h
#define _rtl_mutex_h

#include <rtl/rtl_conf.h>
#include <rtl/rtl_sched.h>

enum {

    PTHREAD_MUTEX_NORMAL,
    PTHREAD_MUTEX_RECURSIVE,
    PTHREAD_MUTEX_ERRORCHECK,
    PTHREAD_MUTEX_DEFAULT =PTHREAD_MUTEX_NORMAL
};

enum {

    PTHREAD_PROCESS_SHARED,
    PTHREAD_PROCESS_PRIVATE
};

enum {

    PTHREAD_PRIO_NONE,
    PTHREAD_PRIO_PROTECT
};

typedef struct {

    int type;
    int pshared;
    int protocol;
    int prioceiling;

} pthread_mutexattr_t;

typedef struct {

    ckhandle_t handle;

} pthread_mutex_t;

#define PTHREAD_MUTEX_INITIALIZER { 0 }

typedef struct {

	int pshared;

} pthread_condattr_t;

typedef struct {

    ckhandle_t handle;

} pthread_cond_t;

#define PTHREAD_COND_INITIALIZER { 0 }

#ifdef __cplusplus
extern "C" {
#endif

#define pthread_mutexattr_init ck_pthread_mutexattr_init
int ck_pthread_mutexattr_init(pthread_mutexattr_t *attr);

#define pthread_mutexattr_settype ck_pthread_mutexattr_settype
int ck_pthread_mutexattr_settype(pthread_mutexattr_t *attr,
				 int type);

#define pthread_mutexattr_gettype ck_pthread_mutexattr_gettype
int ck_pthread_mutexattr_gettype(const pthread_mutexattr_t *attr,
				 int *type);

#define pthread_mutexattr_setpshared ck_pthread_mutexattr_setpshared
int ck_pthread_mutexattr_setpshared(pthread_mutexattr_t *attr,
				    int pshared);

#define pthread_mutexattr_getpshared ck_pthread_mutexattr_getpshared
int ck_pthread_mutexattr_getpshared(const pthread_mutexattr_t *attr,
				    int *pshared);

#define pthread_mutexattr_destroy ck_pthread_mutexattr_destroy
int ck_pthread_mutexattr_destroy(pthread_mutexattr_t *attr);

#define pthread_mutexattr_setprotocol ck_pthread_mutexattr_setprotocol
int ck_pthread_mutexattr_setprotocol(pthread_mutexattr_t *attr,
				     int protocol);

#define pthread_mutexattr_getprotocol ck_pthread_mutexattr_getprotocol
int ck_pthread_mutexattr_getprotocol(const pthread_mutexattr_t *attr,
				     int *protocol);

#define pthread_mutexattr_setprioceiling ck_pthread_mutexattr_setprioceiling
int ck_pthread_mutexattr_setprioceiling(pthread_mutexattr_t *attr,
					int prioceiling);

#define pthread_mutexattr_getprioceiling ck_pthread_mutexattr_getprioceiling
int ck_pthread_mutexattr_getprioceiling(const pthread_mutexattr_t *attr,
					int *prioceiling);

#define pthread_mutex_init ck_pthread_mutex_init
int ck_pthread_mutex_init(pthread_mutex_t *mutex,
			  const pthread_mutexattr_t *attr);

#define pthread_mutex_destroy ck_pthread_mutex_destroy
int ck_pthread_mutex_destroy(pthread_mutex_t *mutex);

#define pthread_mutex_lock ck_pthread_mutex_lock
int ck_pthread_mutex_lock(pthread_mutex_t *mutex);

#define pthread_mutex_trylock ck_pthread_mutex_trylock
int ck_pthread_mutex_trylock(pthread_mutex_t *mutex);

#define pthread_mutex_unlock ck_pthread_mutex_unlock
int ck_pthread_mutex_unlock(pthread_mutex_t *mutex);

#define pthread_mutex_setprioceiling ck_pthread_mutex_setprioceiling
int ck_pthread_mutex_setprioceiling(pthread_mutex_t *mutex,
				    int prioceiling,
				    int *oldceiling);

#define pthread_mutex_getprioceiling ck_pthread_mutex_getprioceiling
int ck_pthread_mutex_getprioceiling(pthread_mutex_t *mutex,
				    int *prioceiling);

#define pthread_condattr_init ck_pthread_condattr_init
int ck_pthread_condattr_init(pthread_condattr_t *attr);

#define pthread_condattr_setpshared ck_pthread_condattr_setpshared
int ck_pthread_condattr_setpshared(pthread_condattr_t *attr,
				   int pshared);

#define pthread_condattr_getpshared ck_pthread_condattr_getpshared
int ck_pthread_condattr_getpshared(const pthread_condattr_t *attr,
				   int *pshared);

#define pthread_condattr_destroy ck_pthread_condattr_destroy
int ck_pthread_condattr_destroy(pthread_condattr_t *attr);

#define pthread_cond_init ck_pthread_cond_init
int ck_pthread_cond_init(pthread_cond_t *cond,
			 const pthread_condattr_t *attr);

#define pthread_cond_destroy ck_pthread_cond_destroy
int ck_pthread_cond_destroy(pthread_cond_t *cond);

#define pthread_cond_wait ck_pthread_cond_wait
int ck_pthread_cond_wait(pthread_cond_t *cond,
			 pthread_mutex_t *mutex);

#define pthread_cond_broadcast ck_pthread_cond_broadcast
int ck_pthread_cond_broadcast(pthread_cond_t *cond);

#define pthread_cond_signal ck_pthread_cond_signal
int ck_pthread_cond_signal(pthread_cond_t *cond);

#define pthread_cond_timedwait ck_pthread_cond_timedwait
int ck_pthread_cond_timedwait(pthread_cond_t *cond,
			      pthread_mutex_t *mutex,
			      const struct timespec *abstime);
#ifdef __cplusplus
};
#endif

#endif /* !_rtl_mutex_h */
