/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Interface to the time management routines.
 * Based on rtl_time.h from the standard RTLinux distribution
 * written by Michael Barabanov <baraban@fsmlabs.com>.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtl_time_h
#define _rtl_time_h

#include <time.h>
#include <ck/stypes.h>
#ifndef IN_CK
#include <rtl/rtl_ckdefs.h>
#endif /* !IN_CK */
#include <rtl/rtl_conf.h>

#define NSECS_PER_SEC   1000000000
#define HRTIME_INFINITY 0x7fffffffffffffffLL

#define HRT_FROM_NS(x)   (x)
#define HRTICKS_PER_SEC  1000000000
#define HRT_FROM_8254(x) ((x) * 838)
#define HRT_TO_8254(x)   ((x) / 838)

typedef hrtime_t RTIME;	/* v1 compat */

enum {

    RTL_CLOCK_MODE_UNINITIALIZED =1,
    RTL_CLOCK_MODE_ONESHOT,
    RTL_CLOCK_MODE_PERIODIC
};

struct pt_regs;
typedef void (*clock_irq_handler_t)(struct pt_regs *);

struct rtl_clock {

    int (*init) (struct rtl_clock *);
    void (*uninit) (struct rtl_clock *);
    hrtime_t (*gethrtime)(struct rtl_clock *);
    int (*sethrtime)(struct rtl_clock *, hrtime_t t);
    int (*settimer)(struct rtl_clock *, hrtime_t interval);
    int (*settimermode)(struct rtl_clock *, int mode);
    clock_irq_handler_t handler;

    int mode;
    hrtime_t resolution;
    hrtime_t value;
    hrtime_t delta;

    ckhandle_t handle;
};

#define timespec_add(t1, t2) { \
	(t1)->tv_nsec += (t2)->tv_nsec;  \
	(t1)->tv_sec += (t2)->tv_sec; \
	if ((t1)->tv_nsec >= NSECS_PER_SEC) { \
		(t1)->tv_nsec -= NSECS_PER_SEC; \
		(t1)->tv_sec++; \
	} else if ((t1)->tv_nsec < 0) { \
		(t1)->tv_nsec += NSECS_PER_SEC; \
		(t1)->tv_sec--; \
	} \
}

#define timespec_sub(t1, t2) { \
	(t1)->tv_nsec -= (t2)->tv_nsec;  \
	(t1)->tv_sec -= (t2)->tv_sec; \
	if ((t1)->tv_nsec < 0) { \
		(t1)->tv_nsec += NSECS_PER_SEC; \
		(t1)->tv_sec--; \
	} else if ((t1)->tv_nsec >= NSECS_PER_SEC) { \
		(t1)->tv_nsec -= NSECS_PER_SEC; \
		(t1)->tv_sec++; \
	} \
}

#define timespec_nz(t) ((t)->tv_sec != 0 || (t)->tv_nsec != 0)

#define timespec_lt(t1, t2) ((t1)->tv_sec < (t2)->tv_sec || \
                             ((t1)->tv_sec == (t2)->tv_sec && \
                              (t1)->tv_nsec < (t2)->tv_nsec))

#define timespec_gt(t1, t2) (timespec_lt(t2, t1))

#define timespec_ge(t1, t2) (!timespec_lt(t1, t2))

#define timespec_le(t1, t2) (!timespec_gt(t1, t2))

#define timespec_eq(t1, t2) ((t1)->tv_sec == (t2)->tv_sec && \
                             (t1)->tv_nsec == (t2)->tv_nsec)

static inline hrtime_t timespec_to_ns (const struct timespec *ts) {

    return (hrtime_t)ts->tv_sec * NSECS_PER_SEC + (hrtime_t)ts->tv_nsec;
}

static inline struct timespec timespec_from_ns (hrtime_t t) {

    struct timespec ts;
    ts.tv_sec = t / NSECS_PER_SEC;
    ts.tv_nsec = t % NSECS_PER_SEC;

    return ts;
}

#ifdef __cplusplus
extern "C" {
#endif

void rtl_delay(long nanoseconds);

int rtl_setclockhandler(clockid_t clock_id,
			clock_irq_handler_t fn);

int rtl_unsetclockhandler(clockid_t clock_id);

clockid_t rtl_getbestclock(unsigned cpu);

#define gethrtime ck_gethrtime
hrtime_t ck_gethrtime(void);

#define gethrtimeres ck_gethrtimeres
hrtime_t ck_gethrtimeres(void);

#ifdef __cplusplus
};
#endif

#endif /* !_rtl_time_h */
