/* -*- C++ -*-
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Interface to the RTLinux-like clock support.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtlClock_h
#define _rtlClock_h

#if defined(__GNUG__) && !defined(__OBSCAN__)
#pragma interface
#endif // __GNUG__ && !__OBSCAN__

#include <ck/stypes.h>
#include "rtlInterrupt.h"
#include "rtl/rtl.h"
#include "rtl/rtl_time.h"

#ifdef IMPLEMENTING_SO
#undef IMPLEMENTING_SO
#endif // IMPLEMENTING_SO
#define IMPLEMENTING_SO IMPLEMENT_RTL
#include "sodecl.h"

class rtlNode;

class rtlSystemTimer : public rtlInterrupt {

protected:

    hrtime_t now,
	resolution;

    clock_irq_handler_t tickHandler;

public:

    rtlSystemTimer(hrtime_t resolution,
		   unsigned irq,
		   rtlNode *node);

    hrtime_t getTime() const {
	return now;
    }

    hrtime_t getRes() const {
	return resolution;
    }

    void setTime(hrtime_t t) {
	now = t;
    }

    void setTimer(enum CfSourceType srcType,
		  hrtime_t interval) {
	resolution = interval;
	configure(srcType,CString().format("%f usc",interval / 1000.0));
    }

    clock_irq_handler_t getTickHandler() {
	return tickHandler;
    }

    void setTickHandler(clock_irq_handler_t fn) {
	tickHandler = fn;
    }

    virtual void kisrt(tick)();
};

class rtlClock : public CkObject {

protected:

    rtlSystemTimer *sysTimer;

    struct rtl_clock *uclock;

public:

    rtlClock(rtlSystemTimer *sysTimer,
	     struct rtl_clock *uclock,
	     rtlNode *node);

    virtual ~rtlClock();

    rtlNode *getNode() {
	return (rtlNode *)CkObject::getNode();
    }

    hrtime_t getRes() const {
	return sysTimer->getRes();
    }

    clock_irq_handler_t getTickHandler() {
	return sysTimer->getTickHandler();
    }

    virtual hrtime_t getTime() const =0;

    ckticks_t getTicks() const {
	return getTime() / getRes();
    }

    virtual void setTime(hrtime_t t) =0;

    virtual void setTimer(enum CfSourceType srcType,
			  hrtime_t interval) =0;

    virtual void setTickHandler(clock_irq_handler_t fn) =0;

    virtual void stop() =0;
};

class rtlClockRealtime : public rtlClock { // == best clock.

public:

    rtlClockRealtime(rtlSystemTimer *sysTimer,
		     struct rtl_clock *uclock,
		     rtlNode *node);

    virtual hrtime_t getTime() const {
	return sysTimer->getTime();
    }

    virtual void setTime(hrtime_t t) {
	sysTimer->setTime(t);
    }

    virtual void setTimer(enum CfSourceType srcType,
			  hrtime_t interval) {
	sysTimer->setTimer(srcType,interval);
    }

    virtual void setTickHandler(clock_irq_handler_t fn) {
	sysTimer->setTickHandler(fn);
    }

    virtual void stop() {
	sysTimer->configure(CfEventNull,NULL);
    }
};

class rtlClockUnadjusted : public rtlClock { // read-only.

public:

    rtlClockUnadjusted(rtlSystemTimer *sysTimer,
		       struct rtl_clock *uclock,
		       rtlNode *node);

    virtual hrtime_t getTime() const {
	return sysTimer->getTime();
    }

    virtual void setTime(hrtime_t t) {
    }

    virtual void setTimer(enum CfSourceType srcType,
			  hrtime_t interval) {
    }

    virtual void setTickHandler(clock_irq_handler_t fn) {
    }

    virtual void stop() {
    }
};

#endif // !_rtlClock_h
