/* -*- C++ -*-
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Interface to the RTLinux FIFO management routines.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtlFifo_h
#define _rtlFifo_h

#if defined(__GNUG__) && !defined(__OBSCAN__)
#pragma interface
#endif // __GNUG__ && !__OBSCAN__

#include <ck/stypes.h>
#include "CString.h"
#include "CkObject.h"
#include "rtl/rtl.h"
#include "rtl/rtl_fifo.h"

#ifdef IMPLEMENTING_SO
#undef IMPLEMENTING_SO
#endif // IMPLEMENTING_SO
#define IMPLEMENTING_SO IMPLEMENT_RTL
#include "sodecl.h"

class CkThread;
class CkInterrupt;
class rtlNode;

class rtlFifo : public CkObject, public SxCallout {

protected:

    unsigned minor;

    CString fifoPath;

    int fd;

    unsigned written : 1,
       readable : 1,
       writable : 1;

    int (*rtHandler)(unsigned int fifo);

    int (*userHandler)(unsigned int fifo);

    void create();

public:

    static void kcout(runHandlers)(rtlNode *node);

    rtlFifo(unsigned minor,
	    rtlNode *node);

    virtual ~rtlFifo();

    rtlNode *getNode() {
	return (rtlNode *)CkObject::getNode();
    }

    void setRtHandler (int (*_rtHandler)(unsigned int fifo)) {
    	rtHandler = _rtHandler;
    }

    void setUserHandler (int (*_userHandler)(unsigned int fifo)) {
    	userHandler = _userHandler;
	written = 0;
    }

    int put(CkRunnable *runnable,
	    const void *buf,
	    int count);

    int get(CkRunnable *runnable,
	    void *buf,
	    int count);

    virtual void process();
};

MakeGList(rtlFifo);

extern rtlFifoGList allFifos;

#endif // !_rtlFifo_h
