/* -*- C++ -*-
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Interface to the RTLinux thread class.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _rtlThread_h
#define _rtlThread_h

#if defined(__GNUG__) && !defined(__OBSCAN__)
#pragma interface
#endif // __GNUG__ && !__OBSCAN__

#include "SxStats.h"
#include "CkThread.h"
#include "CkSynch.h"
#include "rtl/rtl.h"

#ifdef IMPLEMENTING_SO
#undef IMPLEMENTING_SO
#endif // IMPLEMENTING_SO
#define IMPLEMENTING_SO IMPLEMENT_RTL
#include "sodecl.h"

class CkHistogram;
class rtlNode;
class rtlThread;
class rtlMessage;

enum rtlThreadStates {
    rtlThreadHALTED,
    rtlThreadDORMANT,
    rtlThreadDELAYED,
    rtlThreadSLEEPING,
    rtlThreadSUSPENDED,
    rtlThreadREADY,
    rtlThreadACTIVE,
};

#define RTLTHREAD_STACKSIZE 8192 // Default size and internal bare minimum

#define RTL_CANCEL_ENABLED  CK_THREAD_BIT0
#define RTL_CANCEL_DEFERRED CK_THREAD_BIT1
#define RTL_CANCEL_PENDING  CK_THREAD_BIT2
#define RTL_THREAD_JOINABLE CK_THREAD_BIT3
#define RTL_PEND_SENDSIDE   CK_THREAD_BIT4

class rtlThread : public CkThread {

protected:

    void *(*startAddr)(void *arg);

    void *startArg;

    size_t stackSize;

    pthread_t userHandle;

    ITime lastStateChangeTime;

    CkHistogram *stateDistribs[5];

    rtlThread *joiningThread;

    hrtime_t periodStart;

    hrtime_t period;

    int cpuAffinity;

    void logThreadState(rtlThreadStates s);

    virtual void dynamicExpose();

    virtual void dynamicConfigure(StProtoInfoMessage *mbuf);

    void kroot(run)();

    union _wakeUpArgs {

	struct {
	    rtlMessage *msg;
	} queue;

	void *joinValue;

    } wakeUpArgs;

    CkMutex joinMutex;

    CkCondVar joinCv;

public:

    static const char *stateArray[7];

    rtlThread(int priority,
	      void *(*startAddr)(void *arg),
	      void *startArg,
	      size_t stackSize,
	      pthread_t userHandle,
	      const char *name,
	      rtlNode *node);

    virtual ~rtlThread();

    rtlNode *getNode() {
	return (rtlNode *)CkThread::getNode();
    }

    pthread_t getUserHandle() {
	return userHandle;
    }

    void msgSet(rtlMessage *msg) {
	wakeUpArgs.queue.msg = msg;
    }

    rtlMessage *msgGet() const {
	return wakeUpArgs.queue.msg;
    }

    void cleanup();

    void *waitJoin(rtlThread *thread);

    void signalJoin(void *exitValue);

    void setPeriodic(hrtime_t periodStart,
		     hrtime_t period);

    void schedulePeriodic();

    void setCpu(int cpuNum) {
	cpuAffinity = cpuNum;
    }

    int getCpu() const {
	return cpuAffinity;
    }

    virtual void body();

    virtual void setReady();

    virtual void setHalted();

    virtual void setDelayed();

    virtual void setPending();

    virtual void setRunning();

    virtual void setSuspended();

    virtual void setDormant();

    virtual void (*getStartAddr())(...);

    virtual void protoInit();

    virtual void startAsr(cksigmask_t sigs);

    virtual void pipNotify(CkSynchro *synch,
			   CkThread *requestor);

    virtual int pipHold(CkSynchro *synch,
			CkThread *requestor);
};

MakeGList(rtlThread);

class rtlLinuxThread : public rtlThread {

protected:

    SxFlag flag;

public:

    rtlLinuxThread(rtlNode *node);

    void post() {
	flag.post();
    }

    void *khide(doLinux)();

    virtual void dynamicConfigure(StProtoInfoMessage *mbuf);
};

extern rtlThreadGList allRtlThreads;

#endif // !_rtlThread_h
