/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Implementation of the RTLinux interrupt model.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifdef __GNUG__
#pragma implementation
#endif // __GNUG__
#include <config.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include "CkManager.h"
#include "rtlInterrupt.h"
#include "rtlNode.h"

extern "C" {

int rtl_irq_trampoline (ckvector_t vector,
			void *cookie)
{
    rtlInterrupt *interrupt = (rtlInterrupt *)cookie;
    rtl_irq_handler_t handler = interrupt->getHardHandler();

    interrupt->setStatus(CK_INTR_ACKED);
    
    if (handler)
	{
	unsigned irq = interrupt->getLevel() - 1;
	handler(irq,NULL); // No pt_regs under Ck.
	}

    return CK_ISR_DEFERRED;	// Chain the DSR (if any).
}

static void rtl_softirq_trampoline (ckvector_t vector,
				    unsigned count,
				    void *cookie) {

    rtlInterrupt *interrupt = (rtlInterrupt *)cookie;
    rtl_soft_irq_handler_t handler = interrupt->getSoftHandler();
    unsigned irq = interrupt->getLevel() - 1;

    while (handler && count-- > 0)
	// Note: neither dev_id nor pt_regs under Ck.
	handler(irq,NULL,NULL);
}

}

rtlInterrupt::rtlInterrupt (unsigned irq,
			    ckisr_t _trampoline,
			    rtl_irq_handler_t _hardHandler,
			    const char *_name,
			    void *_cookie,
			    rtlNode *_node) :
    CkInterrupt(0,		// Force Ck vector #0.
		irq + 1,	// Ck interrupt level == RTLinux IRQ# + 1
		_trampoline ? _trampoline : &rtl_irq_trampoline,
		&rtl_softirq_trampoline,
		_cookie,
		_name,
		_node) {

    hardHandler = _hardHandler;
    softHandler = NULL;
    setStatus(CK_INTR_MASKED);
    _node->registerIrq(irq,this);
}
