#  This file is part of the CarbonKernel Simulation System.
#  Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
# 
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  Author(s): rpm
# 
#  Contributor(s):

proc rtlMq:attach {context name private} {

    global $context.info
    set $context.info(present) yes
    set $context.info(changed) no
    return [list [list {RTLinux kernel} {Message queues} $name] queue]
}

proc rtlMq:detach {context} {

    global $context.info
    unset $context.info
    catch { destroy $context }
}

proc rtlMq:show {context nid name} {

    if {[winfo exists $context]} {
	wm deiconify $context
	raise $context
    }

    rtlMq:pollInfo $context
}

proc rtlMq:event {context event} {

    switch $event {
	SimulationHeldEvent {
	    if {[catch { wm state $context } state] == 0} {
		if {$state != "withdrawn"} {
		    rtlMq:pollInfo $context
		}
	    }
	}
    }
}

proc rtlMq:update {context nid name info} {

    global $context.info

    # dispatch the information block contents
    # to the per-context data array...
    foreach e $info {
	set item [lindex $e 0]
	set value [lindex $e 1]
	set $context.info($item) $value
    }

    set $context.info(current) $info
    set $context.info(changed) no

    if {[winfo exists $context]} {
	set msgfrm [$context.sw1 subwidget frame]
	set lbox [$msgfrm.list subwidget listbox]
	$lbox delete 0 end

	foreach msg [set $context.info(messages)] {
	    $lbox insert end $msg
	}

	set threadfrm [$context.sw2 subwidget frame]
	rtlThread:fillWaitList $context $threadfrm.list
	return
    }

    # if the buffer object has never been displayed before,
    # build a brand new toplevel for it...

    toplevel $context
    wm title $context "RTLinux mqueue: $name"
    cascadeWindow $context
    wm protocol $context WM_DELETE_WINDOW \
	"wm withdraw $context"

    # Messages

    tixLabelFrame $context.sw1 -label Messages \
	-labelside acrosstop -options {
	    label.padx 5
	}

    pack $context.sw1 -expand true -fill both
    set msgfrm [$context.sw1 subwidget frame]
    tixScrolledListBox $msgfrm.list -scrollbar auto
    pack $msgfrm.list -expand true -fill both
    set lbox [$msgfrm.list subwidget listbox]
    $lbox config -height 5 -width 30 -bd 2

    foreach msg [set $context.info(messages)] {
	$lbox insert end $msg
    }

    # Pending threads

    tixLabelFrame $context.sw2 -label "Pending list" \
	-labelside acrosstop -options {
	    label.padx 5
	}

    pack $context.sw2 -expand true -fill both
    set threadfrm [$context.sw2 subwidget frame]
    tixScrolledListBox $threadfrm.list -scrollbar auto \
	-command "rtlThread:pickWaitList $context $threadfrm.list"
    pack $threadfrm.list -expand true -fill both
    $threadfrm.list subwidget listbox config -height 5 -bd 2
    rtlThread:fillWaitList $context $threadfrm.list

    # Button box

    tixButtonBox $context.bbox -orientation horizontal -relief flat -bd 0
    $context.bbox add update -text Update \
	-command "rtlMq:pollInfo $context"
    $context.bbox add cancel -text Close \
	-command "wm withdraw $context"
    pack $context.bbox -side bottom -fill x
}

proc rtlMq:pollInfo {context} {
    TkRequest $context GetObjectInfo all
}
