/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Testing RTL services:
 * - pthread_create()
 * - pthread_exit()
 * - pthread_self()
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <rtl/rtl.h>

static pthread_t pth1, pth2;

ckhandle_t handle1, handle2;

void *test_thread1 (void *data)

{
    TEST_ASSERT(data == (void *)0xdeadbeef);

    TEST_ASSERT(pthread_self() == pth1);

    TEST_MARK();

    TEST_ASSERT(!ckObjectExists(handle2));

    TEST_CHECK_SEQUENCE(SEQ(handle2,3),
			SEQ(handle1,1),
			END_SEQ);
    TEST_FINISH();

    return NULL;		/* Not executed */
}

void *test_thread2 (void *data)

{
    struct rtl_sched_param param;
    int n = 0;

    TEST_ASSERT(data == (void *)0xbeefdead);

    TEST_ASSERT(pthread_self() == pth2);

    while (n++ < 3)
	TEST_MARK();

    pthread_exit(NULL);

    TEST_MARK();	/* Not executed */

    return NULL;
}

int init_module ()

{
    struct rtl_sched_param param1, param2;
    pthread_attr_t attr1, attr2;

    rtl_sched_lock(pthread_self());

    TEST_START(0);

    param1.sched_priority = 2;
    TEST_ASSERT_OK(pthread_attr_init(&attr1));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr1,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr1,&param1));

    TEST_ASSERT_OK(pthread_create(&pth1,
				  &attr1,
				  test_thread1,
				  (void *)0xdeadbeef));
    handle1 = pth1->handle;

    param2.sched_priority = 3;
    TEST_ASSERT_OK(pthread_attr_init(&attr2));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr2,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr2,&param2));

    TEST_ASSERT_OK(pthread_create(&pth2,
				  &attr2,
				  test_thread2,
				  (void *)0xbeefdead));
    handle2 = pth2->handle;

    rtl_sched_lock(NULL);

    return 0;
}
