/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Testing RTL services:
 * - pthread_attr_init()
 * - pthread_attr_destroy()
 * - pthread_attr_setschedparam()
 * - pthread_attr_getschedparam()
 * - pthread_attr_setstacksize()
 * - sched_get_priority_min()	(Depends on test configuration)
 * - sched_get_priority_max()	(Depends on test configuration)
 * - pthread_attr_getcpu_np
 * - pthread_attr_setcpu_np
 * - pthread_setfp_np
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <rtl/rtl.h>

static struct rtl_thread_struct uthread; /* Forge an invalid thread descriptor */

static pthread_t pth1, upth = &uthread;

ckhandle_t handle1;

void *test_thread1 (void *data)

{
    TEST_MARK();

    TEST_ASSERT(pthread_setfp_np(upth,0) == ESRCH);	/* Error case: invalid thread */
    TEST_ASSERT_OK(pthread_setfp_np(pthread_self(),0));
    TEST_ASSERT_OK(pthread_setfp_np(pthread_self(),1));	/* Null-effect anyway */

    TEST_CHECK_SEQUENCE(SEQ(handle1,1),
			END_SEQ);

    TEST_FINISH();

    return NULL;		/* Not executed */
}

int init_module ()

{
    struct rtl_sched_param param1, param2;
    static pthread_attr_t attr1;
    int cpu = -1;

    rtl_sched_lock(pthread_self());

    TEST_START(0);

    TEST_ASSERT_OK(pthread_attr_init(&attr1));
    TEST_ASSERT(attr1.stack_size > 0);
    TEST_ASSERT(attr1.initial_state == RTL_THREAD_READY);
    TEST_ASSERT(sched_get_priority_min(0) == 0);
    TEST_ASSERT(sched_get_priority_max(0) == 1000000);
    TEST_ASSERT(attr1.sched_param.sched_priority == sched_get_priority_min(0));
    TEST_ASSERT(attr1.cpu == 0);
    TEST_ASSERT(attr1.use_fp == 0);
    TEST_ASSERT_OK(pthread_attr_setcpu_np(&attr1,1));
    TEST_ASSERT_OK(pthread_attr_getcpu_np(&attr1,&cpu));
    TEST_ASSERT(cpu == 1);
    TEST_ASSERT_OK(pthread_attr_setcpu_np(&attr1,0));

    TEST_ASSERT(pthread_attr_setstacksize(&attr1,0) == EINVAL);	/* Error case: invalid stack size */
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr1,8192));
    TEST_ASSERT(attr1.stack_size == 8192);

    param1.sched_priority = sched_get_priority_min(0) - 1;
    TEST_ASSERT(pthread_attr_setschedparam(&attr1,&param1) == EINVAL);	/* Error case: invalid priority */
    param1.sched_priority = sched_get_priority_max(0) + 1;
    TEST_ASSERT(pthread_attr_setschedparam(&attr1,&param1) == EINVAL);	/* Error case: invalid priority */
    param1.sched_priority = 2;
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr1,&param1));
    TEST_ASSERT_OK(pthread_attr_getschedparam(&attr1,&param2));
    TEST_ASSERT(param2.sched_priority == param1.sched_priority);

    TEST_ASSERT_OK(pthread_create(&pth1,
				  &attr1,
				  test_thread1,
				  NULL));
    handle1 = pth1->handle;

    TEST_ASSERT_OK(pthread_attr_destroy(&attr1));

    rtl_sched_lock(NULL);

    return 0;
}
