/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Testing RTL services:
 * - rtf_create()
 * - rtf_destroy()
 * - rtf_put()
 * - rtf_get()
 * - rtf_create_handler()
 * - rtf_create_rt_handler()
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <rtl/rtl.h>

static pthread_t pth1;

ckhandle_t handle1;

int fifo_handler (unsigned fifo)

{
    static int count;
    char buf[16], ok;
    int n;

    n = rtf_get(fifo,buf,sizeof(buf));

    if (count < 2 && n == 0)
	return 0;	/* discard writability statuses */

    switch (count)
	{
	case 0:

	    TEST_ASSERT(n == 6);
	    TEST_ASSERT_OK(strcmp(buf,"hello"));
	    ok = '1';
	    TEST_ASSERT(rtf_put(fifo + 1,&ok,1) == 1);
	    break;

	case 1:

	    TEST_ASSERT(n == 6);
	    TEST_ASSERT_OK(strcmp(buf,"world"));
	    ok = '2';
	    TEST_ASSERT(rtf_put(fifo + 1,&ok,1) == 1);
	    break;

	case 2:

	    TEST_ASSERT(n == 0);
	    break;

	default:

	    TEST_ERROR("unexpected call of fifo_handler()");
	    break;
	}

    count++;

    return 0;
}

void *test_thread1 (void *data)

{
    usleep(10000);

    TEST_FINISH();

    return NULL;		/* Not executed */
}

int init_module ()

{
    struct rtl_sched_param param1;
    pthread_attr_t attr1;
    char *margs[5];

    rtl_sched_lock(pthread_self());

    TEST_START(0);

    param1.sched_priority = 2;
    TEST_ASSERT_OK(pthread_attr_init(&attr1));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr1,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr1,&param1));

    TEST_ASSERT_OK(pthread_create(&pth1,
				  &attr1,
				  test_thread1,
				  NULL));
    handle1 = pth1->handle;

    TEST_ASSERT_OK(rtf_create(0,1024));
    TEST_ASSERT_OK(rtf_create(1,1024));
    TEST_ASSERT_OK(rtf_create_handler(0,fifo_handler));

    margs[0] = "fifomon";
    margs[1] = "/tmp/ckdev/rtf0";
    margs[2] = "/tmp/ckdev/rtf1";
    margs[3] = "a";
    margs[4] = NULL;

    TEST_ASSERT_OK(ckSpawn("&(prefix)/tests/rtl/fifomon",margs));

    rtl_sched_lock(NULL);

    return 0;
}
