/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Testing RTL services:
 * rtl_hard_enable_irq
 * rtl_hard_disable_irq
 * rtl_request_global_irq
 * rtl_free_global_irq
 * rtl_get_soft_irq
 * rtl_free_soft_irq
 * rtl_global_pend_irq
 * rtl_no_interrupts
 * rtl_restore_interrupts
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <rtl/rtl.h>

static pthread_t pth;

int hard_irq_num = 0;

int soft_irq_num;

ckhandle_t thread_handle,
    hard_irq_handle,
    soft_irq_handle;

int hard_irq_handler (unsigned irq, struct pt_regs *pt)

{
    TEST_MARK();
    TEST_ASSERT(irq == (unsigned)hard_irq_num);
    return 0;
}

void soft_irq_handler (unsigned irq, void *arg, struct pt_regs *pt)

{
    TEST_MARK();
    TEST_ASSERT(irq == (unsigned)soft_irq_num);
}

void *test_thread1 (void *data)

{
    rtl_irqstate_t s;

    TEST_MARK();

    rtl_global_pend_irq((unsigned)hard_irq_num);

    rtl_global_pend_irq((unsigned)soft_irq_num);

    TEST_MARK();

    TEST_MARK();

    rtl_no_interrupts(s);

    rtl_global_pend_irq((unsigned)soft_irq_num);

    TEST_MARK();

    TEST_MARK();

    rtl_hard_disable_irq(soft_irq_num);

    rtl_restore_interrupts(s);

    TEST_MARK();

    TEST_MARK();

    rtl_hard_enable_irq(soft_irq_num);

    TEST_CHECK_SEQUENCE(SEQ(thread_handle,1),
			SEQ(hard_irq_handle,1),
			SEQ(soft_irq_handle,1),
			SEQ(thread_handle,6),
			SEQ(soft_irq_handle,1),
			END_SEQ);

    rtl_free_soft_irq(soft_irq_num);

    TEST_FINISH();

    return NULL;		/* Not executed */
}

int init_module ()

{
    extern int rtl_irq_trampoline(ckvector_t vector, void *cookie);
    struct rtl_sched_param param1;
    pthread_attr_t attr1;

    rtl_sched_lock(pthread_self());

    TEST_START(0);

    param1.sched_priority = 3;
    TEST_ASSERT_OK(pthread_attr_init(&attr1));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr1,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr1,&param1));

    TEST_ASSERT_OK(pthread_create(&pth,
				  &attr1,
				  test_thread1,
				  NULL));
    thread_handle = pth->handle;

    TEST_ASSERT_OK(ckCreateIntr("hardIrq",
				0,
				1, /* i.e. IRQ #0 */
				rtl_irq_trampoline,
				NULL,
				NULL,
				&hard_irq_handle));

    TEST_ASSERT_OK(rtl_free_global_irq(hard_irq_num)); /* We need to free it first */

    TEST_ASSERT_OK(rtl_request_global_irq(hard_irq_num,hard_irq_handler));

    rtl_hard_enable_irq((unsigned)hard_irq_num);

    soft_irq_num = rtl_get_soft_irq(soft_irq_handler,"softIrq");

    TEST_ASSERT(soft_irq_num >= 0);

    TEST_ASSERT_OK(ckGetObjectHandle(CK_CURRENT_NID,
				     CK_INTR_OBJECT,
				     "softIrq",
				     &soft_irq_handle));

    rtl_hard_enable_irq((unsigned)soft_irq_num);

    rtl_sched_lock(NULL);

    return 0;
}
