/*
 * This file is part of the CarbonKernel Simulation System.
 * Copyright (C) 1997-2000 Realiant Systems.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The original code is CarbonKernel - Real-time Operating System Simulator,
 * released April 15, 2000. The initial developer of the original code is
 * Realiant Systems (http://www.realiant.com).
 *
 * Description: Testing RTL services:
 * - pthread_mutexattr_setprioceiling()
 * - pthread_mutexattr_setprotocol()
 * - pthread_mutexattr_getprotocol()
 * - pthread_mutexattr_getprioceiling()
 * - pthread_mutex_setprioceiling()
 * - pthread_mutex_getprioceiling()
 * - pthread_mutex_lock()
 * - pthread_mutex_unlock()
 * - rtl_sched_lock()
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#include <ck/testlib.h>
#include <rtl/rtl.h>

static pthread_t pth1, pth2, pth3;

ckhandle_t handle1, handle2, handle3;

pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

void *test_thread1 (void *data)

{
    TEST_MARK();

    usleep(3);

    TEST_MARK();

    TEST_ASSERT_OK(pthread_mutex_lock(&mutex));

    TEST_MARK();

    TEST_ASSERT_OK(pthread_mutex_destroy(&mutex));

    TEST_CHECK_SEQUENCE(SEQ(handle1,1),
			SEQ(handle2,1),
			SEQ(handle3,2),
			SEQ(handle2,1),
			SEQ(handle1,1),
			SEQ(handle3,2),
			SEQ(handle1,1),
			END_SEQ);

    TEST_FINISH();

    return NULL;		/* Not executed */
}

void *test_thread2 (void *data)

{
    TEST_MARK();

    usleep(1);

    TEST_MARK();

    for (;;)
	;

    TEST_MARK();

    return NULL;
}

void *test_thread3 (void *data)

{
    struct rtl_sched_param param;
    int policy, n;

    TEST_MARK();

    TEST_ASSERT_OK(pthread_mutex_lock(&mutex));

    TEST_MARK();

    for (n = 0; n < 30; n++)
	;

    param.sched_priority = -1;
    TEST_ASSERT_OK(pthread_getschedparam(pthread_self(),&policy,&param));
    TEST_ASSERT(param.sched_priority == 3);

    rtl_sched_lock(pthread_self());

    TEST_MARK();

    TEST_ASSERT_OK(pthread_mutex_unlock(&mutex));

    TEST_MARK();

    param.sched_priority = -1;
    TEST_ASSERT_OK(pthread_getschedparam(pthread_self(),&policy,&param));
    TEST_ASSERT(param.sched_priority == 1);

    rtl_sched_lock(NULL);

    rtl_schedule();

    TEST_MARK();

    return NULL;
}

int init_module ()

{
    struct rtl_sched_param param1, param2, param3;
    pthread_attr_t attr1, attr2, attr3;
    pthread_mutexattr_t mattr;
    int mflag;

    rtl_sched_lock(pthread_self());

    TEST_START(0);

    param1.sched_priority = 3;
    TEST_ASSERT_OK(pthread_attr_init(&attr1));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr1,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr1,&param1));

    TEST_ASSERT_OK(pthread_create(&pth1,
				  &attr1,
				  test_thread1,
				  NULL));
    handle1 = pth1->handle;

    param2.sched_priority = 2;
    TEST_ASSERT_OK(pthread_attr_init(&attr2));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr2,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr2,&param2));

    TEST_ASSERT_OK(pthread_create(&pth2,
				  &attr2,
				  test_thread2,
				  NULL));
    handle2 = pth2->handle;

    param3.sched_priority = 1;
    TEST_ASSERT_OK(pthread_attr_init(&attr3));
    TEST_ASSERT_OK(pthread_attr_setstacksize(&attr3,8192));
    TEST_ASSERT_OK(pthread_attr_setschedparam(&attr3,&param3));

    TEST_ASSERT_OK(pthread_create(&pth3,
				  &attr3,
				  test_thread3,
				  NULL));
    handle3 = pth3->handle;

    TEST_ASSERT_OK(pthread_mutexattr_init(&mattr));
    TEST_ASSERT(pthread_mutexattr_setprotocol(&mattr,-1) == EOPNOTSUPP);
    TEST_ASSERT_OK(pthread_mutexattr_setprotocol(&mattr,PTHREAD_PRIO_NONE));
    TEST_ASSERT_OK(pthread_mutexattr_setprotocol(&mattr,PTHREAD_PRIO_PROTECT));
    TEST_ASSERT_OK(pthread_mutexattr_getprotocol(&mattr,&mflag));
    TEST_ASSERT(mflag == PTHREAD_PRIO_PROTECT);
    TEST_ASSERT_OK(pthread_mutexattr_setprioceiling(&mattr,1));
    TEST_ASSERT_OK(pthread_mutexattr_getprioceiling(&mattr,&mflag));
    TEST_ASSERT(mflag == 1);

    TEST_ASSERT_OK(pthread_mutex_init(&mutex,&mattr));
    TEST_ASSERT_OK(pthread_mutex_setprioceiling(&mutex,3,&mflag));
    TEST_ASSERT(mflag == 1);
    TEST_ASSERT_OK(pthread_mutex_getprioceiling(&mutex,&mflag));
    TEST_ASSERT(mflag == 3);

    rtl_sched_lock(NULL);

    return 0;
}
